package cn.ibizlab.api.rest;

import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.math.BigInteger;
import java.util.HashMap;
import lombok.extern.slf4j.Slf4j;
import com.alibaba.fastjson.JSONObject;
import javax.servlet.ServletRequest;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.http.HttpStatus;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.StringUtils;
import org.springframework.context.annotation.Lazy;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import cn.ibizlab.api.dto.*;
import cn.ibizlab.api.mapping.*;
import cn.ibizlab.core.uaa.domain.SYS_AUTHLOG;
import cn.ibizlab.core.uaa.service.ISYS_AUTHLOGService;
import cn.ibizlab.core.uaa.filter.SYS_AUTHLOGSearchContext;

@Slf4j
@Api(tags = {"SYS_AUTHLOG" })
@RestController("api-sys_authlog")
@RequestMapping("")
public class SYS_AUTHLOGResource {

    @Autowired
    private ISYS_AUTHLOGService sys_authlogService;

    @Autowired
    @Lazy
    public SYS_AUTHLOGMapping sys_authlogMapping;

    public SYS_AUTHLOGDTO permissionDTO=new SYS_AUTHLOGDTO();

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibzuaa-SYS_AUTHLOG-GetDraft-all')")
    @ApiOperation(value = "GetDraft", tags = {"SYS_AUTHLOG" },  notes = "GetDraft")
	@RequestMapping(method = RequestMethod.GET, value = "/sys_authlogs/getdraft")
    public ResponseEntity<SYS_AUTHLOGDTO> getDraft() {
        return ResponseEntity.status(HttpStatus.OK).body(sys_authlogMapping.toDto(sys_authlogService.getDraft(new SYS_AUTHLOG())));
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibzuaa-SYS_AUTHLOG-Update-all')")
    @ApiOperation(value = "Update", tags = {"SYS_AUTHLOG" },  notes = "Update")
	@RequestMapping(method = RequestMethod.PUT, value = "/sys_authlogs/{sys_authlog_id}")
    @Transactional
    public ResponseEntity<SYS_AUTHLOGDTO> update(@PathVariable("sys_authlog_id") String sys_authlog_id, @RequestBody SYS_AUTHLOGDTO sys_authlogdto) {
		SYS_AUTHLOG domain = sys_authlogMapping.toDomain(sys_authlogdto);
        domain.setLogid(sys_authlog_id);
		sys_authlogService.update(domain);
		SYS_AUTHLOGDTO dto = sys_authlogMapping.toDto(domain);
        return ResponseEntity.status(HttpStatus.OK).body(dto);
    }

    @PreAuthorize("hasPermission('Update',{'Sql',this.humanMapping,#humandtos})")
    @ApiOperation(value = "UpdateBatch", tags = {"SYS_AUTHLOG" },  notes = "UpdateBatch")
	@RequestMapping(method = RequestMethod.PUT, value = "/sys_authlogs/batch")
    public ResponseEntity<Boolean> updateBatch(@RequestBody List<SYS_AUTHLOGDTO> sys_authlogdtos) {
        sys_authlogService.updateBatch(sys_authlogMapping.toDomain(sys_authlogdtos));
        return  ResponseEntity.status(HttpStatus.OK).body(true);
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibzuaa-SYS_AUTHLOG-Remove-all')")
    @ApiOperation(value = "Remove", tags = {"SYS_AUTHLOG" },  notes = "Remove")
	@RequestMapping(method = RequestMethod.DELETE, value = "/sys_authlogs/{sys_authlog_id}")
    @Transactional
    public ResponseEntity<Boolean> remove(@PathVariable("sys_authlog_id") String sys_authlog_id) {
         return ResponseEntity.status(HttpStatus.OK).body(sys_authlogService.remove(sys_authlog_id));
    }

    @PreAuthorize("hasPermission('Remove',{'Sql',this.humanMapping,this.permissionDTO,#ids})")
    @ApiOperation(value = "RemoveBatch", tags = {"SYS_AUTHLOG" },  notes = "RemoveBatch")
	@RequestMapping(method = RequestMethod.DELETE, value = "/sys_authlogs/batch")
    public ResponseEntity<Boolean> removeBatch(@RequestBody List<String> ids) {
        sys_authlogService.removeBatch(ids);
        return  ResponseEntity.status(HttpStatus.OK).body(true);
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibzuaa-SYS_AUTHLOG-Create-all')")
    @ApiOperation(value = "Create", tags = {"SYS_AUTHLOG" },  notes = "Create")
	@RequestMapping(method = RequestMethod.POST, value = "/sys_authlogs")
    @Transactional
    public ResponseEntity<SYS_AUTHLOGDTO> create(@RequestBody SYS_AUTHLOGDTO sys_authlogdto) {
        SYS_AUTHLOG domain = sys_authlogMapping.toDomain(sys_authlogdto);
		sys_authlogService.create(domain);
        SYS_AUTHLOGDTO dto = sys_authlogMapping.toDto(domain);
		return ResponseEntity.status(HttpStatus.OK).body(dto);
    }

    @PreAuthorize("hasPermission('Create',{'Sql',this.humanMapping,#humandtos})")
    @ApiOperation(value = "createBatch", tags = {"SYS_AUTHLOG" },  notes = "createBatch")
	@RequestMapping(method = RequestMethod.POST, value = "/sys_authlogs/batch")
    public ResponseEntity<Boolean> createBatch(@RequestBody List<SYS_AUTHLOGDTO> sys_authlogdtos) {
        sys_authlogService.createBatch(sys_authlogMapping.toDomain(sys_authlogdtos));
        return  ResponseEntity.status(HttpStatus.OK).body(true);
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibzuaa-SYS_AUTHLOG-CheckKey-all')")
    @ApiOperation(value = "CheckKey", tags = {"SYS_AUTHLOG" },  notes = "CheckKey")
	@RequestMapping(method = RequestMethod.POST, value = "/sys_authlogs/checkkey")
    public ResponseEntity<Boolean> checkKey(@RequestBody SYS_AUTHLOGDTO sys_authlogdto) {
        return  ResponseEntity.status(HttpStatus.OK).body(sys_authlogService.checkKey(sys_authlogMapping.toDomain(sys_authlogdto)));
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibzuaa-SYS_AUTHLOG-Get-all')")
    @ApiOperation(value = "Get", tags = {"SYS_AUTHLOG" },  notes = "Get")
	@RequestMapping(method = RequestMethod.GET, value = "/sys_authlogs/{sys_authlog_id}")
    public ResponseEntity<SYS_AUTHLOGDTO> get(@PathVariable("sys_authlog_id") String sys_authlog_id) {
        SYS_AUTHLOG domain = sys_authlogService.get(sys_authlog_id);
        SYS_AUTHLOGDTO dto = sys_authlogMapping.toDto(domain);
        return ResponseEntity.status(HttpStatus.OK).body(dto);
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibzuaa-SYS_AUTHLOG-Save-all')")
    @ApiOperation(value = "Save", tags = {"SYS_AUTHLOG" },  notes = "Save")
	@RequestMapping(method = RequestMethod.POST, value = "/sys_authlogs/save")
    public ResponseEntity<Boolean> save(@RequestBody SYS_AUTHLOGDTO sys_authlogdto) {
        return ResponseEntity.status(HttpStatus.OK).body(sys_authlogService.save(sys_authlogMapping.toDomain(sys_authlogdto)));
    }

    @PreAuthorize("hasPermission('Save',{'Sql',this.humanMapping,#humandtos})")
    @ApiOperation(value = "SaveBatch", tags = {"SYS_AUTHLOG" },  notes = "SaveBatch")
	@RequestMapping(method = RequestMethod.POST, value = "/sys_authlogs/savebatch")
    public ResponseEntity<Boolean> saveBatch(@RequestBody List<SYS_AUTHLOGDTO> sys_authlogdtos) {
        sys_authlogService.saveBatch(sys_authlogMapping.toDomain(sys_authlogdtos));
        return  ResponseEntity.status(HttpStatus.OK).body(true);
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibzuaa-SYS_AUTHLOG-Default-all')")
	@ApiOperation(value = "fetchDEFAULT", tags = {"SYS_AUTHLOG" } ,notes = "fetchDEFAULT")
    @RequestMapping(method= RequestMethod.GET , value="/sys_authlogs/fetchdefault")
	public ResponseEntity<List<SYS_AUTHLOGDTO>> fetchDefault(SYS_AUTHLOGSearchContext context) {
        Page<SYS_AUTHLOG> domains = sys_authlogService.searchDefault(context) ;
        List<SYS_AUTHLOGDTO> list = sys_authlogMapping.toDto(domains.getContent());
        return ResponseEntity.status(HttpStatus.OK)
                .header("x-page", String.valueOf(context.getPageable().getPageNumber()))
                .header("x-per-page", String.valueOf(context.getPageable().getPageSize()))
                .header("x-total", String.valueOf(domains.getTotalElements()))
                .body(list);
	}

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibzuaa-SYS_AUTHLOG-Default-all')")
	@ApiOperation(value = "searchDEFAULT", tags = {"SYS_AUTHLOG" } ,notes = "searchDEFAULT")
    @RequestMapping(method= RequestMethod.POST , value="/sys_authlogs/searchdefault")
	public ResponseEntity<Page<SYS_AUTHLOGDTO>> searchDefault(@RequestBody SYS_AUTHLOGSearchContext context) {
        Page<SYS_AUTHLOG> domains = sys_authlogService.searchDefault(context) ;
	    return ResponseEntity.status(HttpStatus.OK)
                .body(new PageImpl(sys_authlogMapping.toDto(domains.getContent()), context.getPageable(), domains.getTotalElements()));
	}
}
